/*
*   PlotPanel  -- A Swing panel that displays the specified data plot.
*
*   Copyright (C) 2000-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
**/
package jahuwaldt.plot;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import com.touchgraph.graphlayout.*;
import java.awt.image.*;
import javax.imageio.*;

/**
*  <p> A panel component that allows the supplied data
*      plot to be included and drawn in a Swing container.
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author  Joseph A. Huwaldt   Date:  June 1, 2000
*  @version November 18, 2000
**/
public class PlotPanel extends JPanel implements MouseListener, MouseMotionListener {

	private int x1 = 0;			// coordinates for selection of map area
	private int y1 = 0;
	private int x2 = 0;
	private int y2 = 0;
	private int x2old = 0;
	private int y2old = 0;
    private int jobnbr;

	/**
	*  The plot this panel is displaying
	**/
	private SimplePlotXY thePlot;
        public Rectangle plotBounds;
    private double xUpper;
    private double xLower;
    private double yUpper;
    private double yLower;
        private double [][] solution;
        private int N;
        private int plotTag;
    private int size;
    public boolean visualize = false;
    

	   
  
	//-------------------------------------------------------------------------
	/**
	*  Creates a plot panel that displays the given plot.
	*
	*  @param  plot   The plot to be displayed in this panel.
	**/
	public PlotPanel( SimplePlotXY plot, double[][] solcol, int soln, int tag, int s, int nbr ) {
		super();

		thePlot = plot;
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		solution = solcol;
		N = soln;
		plotTag = tag;
		size = s;
		jobnbr = nbr;
	}

	public PlotPanel( SimplePlotXY plot) {
		super();

		thePlot = plot;
	}

	//-------------------------------------------------------------------------
	/**
	*  Returns a reference to the plot displayed in this
	*  panel.
	**/
	public SimplePlotXY getPlot() {
		return thePlot;
	}
	
	/**
	*  Called automatically by system to paint this component.
	**/
	public void paintComponent( Graphics gc ) {

		// Paint the background
		super.paintComponent( gc );


		
		if (thePlot != null) {
			// Deal with a possible border.
			Insets insets = getInsets();
			int currentWidth = getWidth() - insets.left - insets.right;
			int currentHeight = getHeight() - insets.top - insets.bottom;
			Rectangle bounds = new Rectangle(insets.left, insets.top,currentWidth, currentHeight);
			
			//	Render the plot image.
			thePlot.draw(gc, this, bounds);
			plotBounds = thePlot.plotBounds;
			xUpper = thePlot.xUpper;
			yUpper = thePlot.yUpper;
			xLower = thePlot.xLower;
			yLower = thePlot.yLower;
			//			System.out.println(plotBounds.x + " " + plotBounds.y + " " + plotBounds.width + " " + plotBounds.height);


		} 
	}

  


	/** Load document data */
	public void mouseClicked(MouseEvent e) {
	
	    if (N==0) return;

		int x = e.getX();
		int y = e.getY();
		
		double X = (double)(x-plotBounds.x)/(double)plotBounds.width;
		double Y = 1.0-(double)(y-plotBounds.y)/(double)(plotBounds.height);

		if (X < 0 ||  X > 1 || Y < 0 || Y > 1) return;
	
		X = xLower+(double)(X)*(xUpper-xLower);
		Y = yLower+(double)(Y)*(yUpper-yLower);
	
		double mindist = 1e100;
		int mini = -1;


		// Pareto front plot
		if (plotTag == 1) {

		
		    for (int i=0; i<N; i++) {
			double dist = Math.sqrt((solution[i][2]-X)*(solution[i][2]-X)+(solution[i][3]-Y)*(solution[i][3]-Y));
			mindist = Math.min(dist, mindist);
			if (dist == mindist) {
			    mini = i;
			}
		    }
		   

		}
		// Attainment score plot
		else if (plotTag == 2) {
		   
		    for (int i=0; i<N; i++) {
			double dist = Math.sqrt((solution[i][1]-X)*(solution[i][1]-X)+(solution[i][4]-Y)*(solution[i][4]-Y));
			mindist = Math.min(dist, mindist);
			if (dist == mindist) {
			    mini = i;
			}
		    }
		    
		}

		if (mini == -1) {
		    System.err.println("No solutions available " + N);
		    return;
		}
		
	        try {

		    if (visualize == false) {
			final String helpstring = 
			    "This is solution " + jobnbr+"-"+(int)(solution[mini][0]) + ".solution.\n" +
			    "It contains k="+ (int)(solution[mini][1]) + " clusters.\n" +
			    "Normalized objective values are connectivity=" + solution[mini][2] + ", overall deviation=" + solution[mini][3] + ".\n" +
			    "Its attainment score is " + solution[mini][4] + ".\n" +
			    "Its F-Measure value is " + solution[mini][5] + ".\n" +
			    "Activate graph layout for a more detailed visualization.\n";
			JOptionPane.showMessageDialog(null, helpstring, "Solution information", JOptionPane.INFORMATION_MESSAGE, null);

		    }

		    else if (visualize == true) {


			String s = new String(jobnbr+"-"+(int)(solution[mini][0]) + ".all [k=" + (int)(solution[mini][1]) + ", connectivity=" + solution[mini][2] + ", overall deviation=" + solution[mini][3] + ", attainment score=" + solution[mini][4] + "]");
			
			String name2 = new String(jobnbr+"-"+(int)(solution[mini][0])+".clust");
			BufferedReader in2 = new BufferedReader(new FileReader(name2));
			String line = in2.readLine();
			int K = new Integer(line).intValue();
			double [][] dist = new double[K][K];
			for (int i=0; i<K; i++) {
			    for (int j=0; j<i; j++) {
				line = in2.readLine();
				dist[i][j] = new Double(line).doubleValue();
			    }
			}
			
			
			String name = new String(jobnbr+"-"+(int)(solution[mini][0])+".all");	    
			BufferedReader in = new BufferedReader(new FileReader(name));
			int N = size;
			int [] ass = new int[N];
			double [] strength = new double[N];
			String [] labels = new String[N];
			int ctr = 0;
			
			while ((line = in.readLine()) != null) {
			    int start = 0;
			    int end = line.indexOf(' ');
			    labels[ctr] = line.substring(start, end);
			    line = line.substring(end+1);
			    end = line.indexOf(' ');
			    ass[ctr] = new Integer(line.substring(start,end)).intValue();
			    line = line.substring(end+1);
			    strength[ctr] = new Double(line).doubleValue();
			    ctr++;
			}
			
			
			JFrame newframe = new JFrame(s);
			final GLPanel p = new GLPanel(Color.white,labels,ass,strength,N, dist, K);
			p.setSize(800,700);
			MenuBar mainBar = new MenuBar();
			Menu plot = new Menu("Plot");
			MenuItem save = new MenuItem("Save plot in .jpg format");
			save.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
				    
				    
				    try {
					JFileChooser chooser = new JFileChooser(".");
					JFrame frame = new JFrame();
					int returnVal = chooser.showSaveDialog(frame);
					String filename = "";
					if (returnVal == JFileChooser.APPROVE_OPTION) {
					    filename = chooser.getSelectedFile().getAbsolutePath();
					    File f = new File(filename);
					    
					    BufferedImage image = new BufferedImage(p.getWidth(), p.getHeight(), BufferedImage.TYPE_INT_RGB);
					    Graphics2D g = image.createGraphics();
					    g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,RenderingHints.VALUE_FRACTIONALMETRICS_ON);
					    g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
					    p.paint(g);
					    g.dispose();
					    ImageIO.write(image, "jpeg", f);
					    image.flush();
					    
					}
					
				    } catch (Exception ee) {
					System.err.println("Error while trying to save plots");
					System.err.println(ee.getMessage());
					ee.printStackTrace();
				    }
				}});
			plot.add(save);
			mainBar.add(plot);
			
			newframe.setMenuBar(mainBar);
			newframe.getContentPane().add(p);
			newframe.pack();
			newframe.setSize(800,700);
			newframe.setLocation(200,100);
			newframe.show();
			newframe.setVisible(true);
		    }
		} catch (Exception ee) {
		    System.err.println("Error while trying to read solution");
		    
System.err.println(ee.getMessage());
		    ee.printStackTrace();
		}
		


	}
	/** Start area selection */
	public void mousePressed(MouseEvent e) {
		x1 = (int)Math.floor(e.getX());
		y1 = (int)Math.floor(e.getY());
		x2 = x1;
		y2 = y1;
		x2old = x1;
		y2old = y2;
	}
	public void mouseEntered(MouseEvent e) {
	}

	/** Finish area selection - Load document data*/
	public void mouseReleased(MouseEvent e) {
	    if (N==0) return;

		try {
			Graphics gr = this.getGraphics();
			x2 = (int)Math.floor(e.getX());
			y2 = (int)Math.floor(e.getY());
			if ((x1 == x2) && (y1 == y2)) return;
			gr.setColor(Color.red);
			gr.drawRect(Math.min(x1, x2), Math.min(y1, y2), Math.abs(x2-x1), Math.abs(y2-y1));

			double X1 = (double)(x1-plotBounds.x)/(double)plotBounds.width;
			double Y1 = 1.0-(double)(y1-plotBounds.y)/(double)(plotBounds.height);
			double X2 = (double)(x2-plotBounds.x)/(double)plotBounds.width;
			double Y2 = 1.0-(double)(y2-plotBounds.y)/(double)(plotBounds.height);

			if (X1 < 0 || X1 > 1 || X2 < 0 || X2 > 1 || Y1 < 0 || Y1 > 1 || Y2 < 0 || Y2 > 1) return;

			X1 = xLower+(double)(X1)*(xUpper-xLower);
			Y1 = yLower+(double)(Y1)*(yUpper-yLower);
			X2 = xLower+(double)(X2)*(xUpper-xLower);
			Y2 = yLower+(double)(Y2)*(yUpper-yLower);

			if (X1 > X2) {
			    double temp = X2;
			    X2 = X1;
			    X1 = temp;
			}
			if (Y1 > Y2) {
			    double temp = Y2;
			    Y2 = Y1;
			    Y1 = temp;
			}

			int cctr =0;

			if (plotTag == 1) {
			  	    for (int i=0; i<N; i++) {


				if ((solution[i][2] >= X1) && (solution[i][2] <= X2) && (solution[i][3] >= Y1) &&  (solution[i][3] <= Y2)) {
				    try {
				
					String s = new String(jobnbr+"-"+(int)(solution[i][0]) + ".all [k=" + (int)(solution[i][1]) + ", connectivity=" + solution[i][2] + ", overall deviation=" + solution[i][3] + ", attainment score=" + solution[i][4] + "]");

					String name2 = new String(jobnbr+"-"+(int)(solution[i][0])+".clust");
					BufferedReader in2 = new BufferedReader(new FileReader(name2));
					String line = in2.readLine();
					int K = new Integer(line).intValue();
					double [][] dist = new double[K][K];
					for (int l=0; l<K; l++) {
					    for (int j=0; j<l; j++) {
						line = in2.readLine();
						dist[l][j] = new Double(line).doubleValue();
					    }
					}

					String name = new String(jobnbr+"-"+(int)(solution[i][0])+".all");
					BufferedReader in = new BufferedReader(new FileReader(name));
					int N = size;
					int [] ass = new int[N];
					double [] strength = new double[N];
					String [] labels = new String[N];
					int ctr = 0;
					
					while ((line = in.readLine()) != null) {

					    int start = 0;
					    int end = line.indexOf(' ');
					    labels[ctr] = line.substring(start, end);
					    line = line.substring(end+1);
					    end = line.indexOf(' ');
					    ass[ctr] = new Integer(line.substring(start,end)).intValue();
					    line = line.substring(end+1);
					    strength[ctr] = new Double(line).doubleValue();
					    ctr++;
					}

				
					JFrame newframe = new JFrame(s);
					final GLPanel p = new GLPanel(Color.white,labels,ass,strength,N, dist, K);
					p.setSize(800,700);
					MenuBar mainBar = new MenuBar();
					Menu plot = new Menu("Plot");
					mainBar.add(plot);
					MenuItem save = new MenuItem("Save plot in .jpg format");
					save.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {

						 					    
						    try {
							JFileChooser chooser = new JFileChooser(".");
							JFrame frame = new JFrame();
							int returnVal = chooser.showSaveDialog(frame);
							String filename = "";
							if (returnVal == JFileChooser.APPROVE_OPTION) {
							    filename = chooser.getSelectedFile().getAbsolutePath();
							    File f = new File(filename);
							    
							    BufferedImage image = new BufferedImage(p.getWidth(), p.getHeight(), BufferedImage.TYPE_INT_RGB);
							    Graphics2D g = image.createGraphics();
							    g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,RenderingHints.VALUE_FRACTIONALMETRICS_ON);
							    g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
							    p.paint(g);
							    g.dispose();
							    ImageIO.write(image, "jpeg", f);
							    image.flush();
	    
							}
							
						    } catch (Exception ee) {
							System.err.println("Error while trying to save plots");
							System.err.println(ee.getMessage());
							ee.printStackTrace();
						    }
						}});
                 		        plot.add(save);
					newframe.setMenuBar(mainBar);
					newframe.getContentPane().add(p);
					newframe.pack();
					newframe.setSize(800,700);
					newframe.setLocation(200+cctr*10,100+cctr*10);
					newframe.show();
					newframe.setVisible(true);
				    } catch (Exception ee) {
					System.err.println("Error while trying to read solution");
					System.err.println(ee.getMessage());
					ee.printStackTrace();
				    }
				    cctr++;
				}
			    }
			 
			}

			else if (plotTag ==2) {
			        for (int i=0; i<N; i++) {
				if ((solution[i][1] >= X1) && (solution[i][1] <= X2) && (solution[i][4] >= Y1) &&  (solution[i][4] <= Y2)) {
			     try {
				String s = new String(jobnbr+"-"+(int)(solution[i][0]) + ".solution [k=" + (int)(solution[i][1]) + ", connectivity=" + solution[i][2] + ", overall deviation=" + solution[i][3] + ", attainment score=" + solution[i][4] + "]");


				String name2 = new String(jobnbr+"-"+(int)(solution[i][0])+".clust");
				BufferedReader in2 = new BufferedReader(new FileReader(name2));
				String line = in2.readLine();
				int K = new Integer(line).intValue();
				double [][] dist = new double[K][K];
				for (int l=0; l<K; l++) {
				    for (int j=0; j<l; j++) {
					line = in2.readLine();
					dist[l][j] = new Double(line).doubleValue();
				    }
				}
				String name = new String(jobnbr+"-"+(int)(solution[i][0])+".all");
					BufferedReader in = new BufferedReader(new FileReader(name));
					int N = size;
					int [] ass = new int[N];
					double [] strength = new double[N];
					String [] labels = new String[N];
					int ctr = 0;
					
					while ((line = in.readLine()) != null) {
					    int start = 0;
					    int end = line.indexOf(' ');
					    labels[ctr] = line.substring(start, end);
					    line = line.substring(end+1);
					    end = line.indexOf(' ');
					    ass[ctr] = new Integer(line.substring(start,end)).intValue();
					    line = line.substring(end+1);
					    strength[ctr] = new Double(line).doubleValue();
					    ctr++;
					}

					
					JFrame newframe = new JFrame(s);
					final GLPanel p = new GLPanel(Color.white,labels,ass,strength,N, dist, K);
					p.setSize(800,700);

					
					MenuBar mainBar = new MenuBar();
					Menu plot = new Menu("Plot");
					mainBar.add(plot);
					MenuItem save = new MenuItem("Save plot in .jpg format");
					save.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {

						 					    
						    try {
							JFileChooser chooser = new JFileChooser(".");
							JFrame frame = new JFrame();
							int returnVal = chooser.showSaveDialog(frame);
							String filename = "";
							if (returnVal == JFileChooser.APPROVE_OPTION) {
							    filename = chooser.getSelectedFile().getAbsolutePath();
							    File f = new File(filename);
							    
							    BufferedImage image = new BufferedImage(p.getWidth(), p.getHeight(), BufferedImage.TYPE_INT_RGB);
							    Graphics2D g = image.createGraphics();
							    g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,RenderingHints.VALUE_FRACTIONALMETRICS_ON);
							    g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
							    p.paint(g);
							    g.dispose();
							    ImageIO.write(image, "jpeg", f);
							    image.flush();
	    
							}
							
						    } catch (Exception ee) {
							System.err.println("Error while trying to save plots");
							System.err.println(ee.getMessage());
							ee.printStackTrace();
						    }
						}});
                 		        plot.add(save);
			                newframe.setMenuBar(mainBar);
					newframe.getContentPane().add(p);
					newframe.pack();
					newframe.setSize(800,700);
					newframe.setLocation(200+cctr*10,100+cctr*10);
					newframe.show();
					newframe.setVisible(true);
				    } catch (Exception ee) {
					System.err.println("Error while trying to read solution");
					System.err.println(ee.getMessage());
					ee.printStackTrace();
				    }
			    cctr++;
				}
			    }
			     
			}



			this.repaint();
		} catch (Exception exc) {}
		
	}
	public void mouseExited(MouseEvent e) {
	}

	/** Update the rectangle for area selection */
	public void mouseDragged(MouseEvent e) {
		x2old = x2;
		y2old = y2;
		x2 = (int)Math.floor(e.getX());
		y2 = (int)Math.floor(e.getY());
		if (((x1 < x2) && (x2 < x2old)) || ((x1 > x2) && (x2 > x2old))) {
			this.repaint();
		}
		if (((y1 < y2) && (y2 < y2old)) || ((y1 > y2) && (y2 > y2old))) {
			this.repaint();
		}
		Graphics gr = this.getGraphics();
		gr.setColor(Color.red);
		gr.drawLine(x1, y1, x1, y2);
		gr.drawLine(x1, y1, x2, y1);
		
	}
	public void mouseMoved(MouseEvent e) {
	}




}


